
from ..qt_compat import QtGui, QtCore

import numpy as np
import logging as log

import os
import matplotlib

from ..plugins import Plugin
from ..core import DataModel, LayerManager, LabelManager, Launcher
from .mpl_widgets import PerspectiveCanvas
from .base import SComboBox

class ConfidenceViewer(Plugin):

    name = 'Confidence Viewer'

    def __init__(self, ptype=Plugin.Widget):
        super(ConfidenceViewer, self).__init__(ptype=ptype)
        self.DM = DataModel.instance()
        self.LBLM = LabelManager.instance()
        self.DM.confidence_changed.connect(self.replot)
        vbox = QtWidgets.QVBoxLayout()
        self.layout.addLayout(vbox, 0, 0)

        self.orient = 0
        self.canvases = (None,)
        self.idx = (self.DM.data.shape[self.orient]) // 2

        topbox = QtWidgets.QHBoxLayout()
        vbox.addLayout(topbox, 0)
        # Perspective
        self.combo = SComboBox()
        self.combo.addItem("Axial")
        self.combo.addItem("Sagittal")
        self.combo.addItem("Coronal")
        self.combo.setCurrentIndex(0)
        self.combo.currentIndexChanged.connect(self.on_combo)
        topbox.addWidget(self.combo)

        # Slider
        self.slider = QtWidgets.QSlider(1)
        self.slider.setMinimum(0)
        self.slider.setMaximum(self.DM.data.shape[0]-1)
        self.slider.setValue(self.idx)
        self.slider.valueChanged.connect(self.on_value)
        self.slider.setSingleStep(1)
        topbox.addWidget(self.slider)

        # Slider Text
        self.text_idx = QtWidgets.QLabel(str(self.idx))
        topbox.addWidget(self.text_idx)

        self.container = QtWidgets.QWidget()
        self.container.setLayout(QtGui.QGridLayout())
        vbox.addWidget(self.container, 1)

    def on_value(self, idx):
        self.idx = idx
        self.text_idx.setText(str(self.idx))

    def clear(self):
        for widget in self.canvases:
            if widget is not None:
                widget.setParent(None)
                widget.deleteLater()
        self.canvases = (None,)

    def on_combo(self, orient):
        self.orient = orient
        self.idx = (self.DM.data.shape[self.orient]) // 2
        for widget in self.canvases:
            if widget is not None:
                widget.idx = self.idx
                widget.orient = orient
                widget.replot()

    def replot(self):
        self.clear()
        total = self.LBLM.len()
        spunique = np.unique(self.DM.gtlabels[:])
        cols = total if total < 4 else total//2 if total%2==0 else total//2+1
        rows = total//cols if total%cols==0 else total//cols+1
        k = 0
        labels = list(self.LBLM.labels())
        self.canvases = []
        for i in range(rows):
            for j in range(cols):
                name = None if i*cols+j >= len(labels) else labels[i*cols+j].name
                if k in spunique:
                    widget = ConfidenceCanvas(k, name=name)
                    k += 1
                else:
                    widget = ConfidenceCanvas(name=name)
                self.canvases += [widget]
                self.slider.valueChanged.connect(widget.update_volume)
                self.container.layout().addWidget(widget, i, j)





import numpy as np
import pandas as pd

from ..qt_compat import QtGui, QtCore, QtWidgets

from .mpl_widgets import PerspectiveCanvas, MplCanvas
from .base import HWidgets, TComboBox, RoundedWidget, ColorButton, PLineEdit, \
                  LCheckBox, SComboBox, CheckableLabels, SourceCombo

from .. import actions as ac

from ..plugins.base import Plugin
from ..core import DataModel, LabelManager, Launcher, LayerManager


import logging as log
from collections import OrderedDict

import seaborn as sns

from .label_partitioning import FEATURE_TYPES, FEATURE_OPTIONS
from six import iteritems

class LevelStats(Plugin):

    name = 'Level Statistics'

    def __init__(self, ptype=Plugin.Widget):
        super(LevelStats, self).__init__(ptype=ptype)

        self.DM = DataModel.instance()
        self.LBLM = LabelManager.instance()

        self.level_combo = SComboBox()
        self.level_combo.setMinimumWidth(200)
        self.label_combo = CheckableLabels(-1)
        self.label_combo.setMinimumWidth(200)
        self.source_combo = SourceCombo()
        self.source_combo.setMinimumWidth(200)
        self.label_btn = QtWidgets.QPushButton('Label')
        self.addWidget(HWidgets(self.level_combo, self.label_combo,
                                self.source_combo, self.label_btn,
                                stretch=[1, 1, 1, 1]))

        self.mplcanvas = MplCanvas(axisoff=False, autoscale=True)
        self.mplcanvas.ax.set_yticks([])

        self.kernels = TComboBox('Fit kernel:', ['gau', 'cos', 'biw', 'epa',
                                                'tri', 'triw'],
                                 selected=0)
        self.update_plot = QtWidgets.QPushButton('Update plot')
        self.export_plot = QtWidgets.QPushButton('Export plot')
        self.export_stats = QtWidgets.QPushButton('Export Stats')
        self.addWidget(HWidgets(self.kernels, None, self.update_plot,
                                self.export_plot, self.export_stats,
                                stretch=[0, 1, 0, 0]))

        splitter = QtWidgets.QSplitter(1)
        self.addWidget(splitter)

        group = QtWidgets.QGroupBox()
        vbox = QtWidgets.QVBoxLayout()
        group.setLayout(vbox)

        self.options = OrderedDict()

        for name, define in zip(FEATURE_OPTIONS, FEATURE_TYPES):
            dummy = LCheckBox(name)
            dummy.setChecked(False)
            self.options[define] = (name, dummy)
            vbox.addWidget(dummy)

        splitter.addWidget(self.mplcanvas)
        splitter.addWidget(group)

        self.LBLM.levelLoaded.connect(self.on_level_added)
        self.LBLM.levelAdded.connect(self.on_level_added)
        self.LBLM.levelRemoved.connect(self.on_level_removed)
        self.level_combo.currentIndexChanged.connect(self.on_level_changed)
        self.label_btn.clicked.connect(self.on_label_clicked)

        self.update_plot.clicked.connect(self.on_update_plot)
        self.export_plot.clicked.connect(self.on_export_plot)
        self.export_stats.clicked.connect(self.on_export_stats)
        self.kernels.currentIndexChanged.connect(self.on_kernel_changed)

        self.levels = []
        self.selected_label = None
        self.kernel = 'gau'
        self.feature = None
        self.colors = None
        self.color_idx = None
        self.labels = None
        self.objects = None
        self.computed_level = None

    def on_level_changed(self, index):
        self.label_combo.clear()

        if index < 0:
            return

        self.label_combo.selectLevel(self.levels[index])

    def on_level_added(self, level):
        self.levels.append(level)
        self.level_combo.addItem('Level {}'.format(level))
        if level == 0:
            self.on_level_changed(0)
            self.label_combo.selectLevel(level)

    def on_level_removed(self, level):
        idx = self.levels.index(level)
        self.level_combo.removeItem(idx)
        if self.computed_level is not None and level == self.computed_level:
            self.reset()

    def on_label_clicked(self):
        self.reset()

        source = self.source_combo.value()
        level = self.levels[self.level_combo.currentIndex()]
        self.computed_level = level
        self.LBLM.save(level)

        labels = self.label_combo.getSelectedLabels()

        out_features = []
        for f in FEATURE_TYPES:
            out_features.append('objects/{}'.format(f))

        in_dset = self.LBLM.dataset(level)
        out_dset = 'objects/objects'
        self.DM.remove_dataset(out_dset)

        Launcher.instance().run(ac.label_objects, dataset=in_dset, source=source,
                                out=out_dset, out_features=out_features,
                                labels=labels, caption='Labelling objects...',
                                return_labels=True, cb=self.on_objects_labelled)

    def on_objects_labelled(self, params):
        self.objects, self.num_objects, self.labels = params
        if self.num_objects == 0:
            QtWidgets.QMessageBox.critical(self, "Error", 'No objects found')
            return
        self.on_update_plot()


    def on_update_plot(self, res=None):
        if self.objects is None:
            QtWidgets.QMessageBox.critical(self, 'Error', 'No objects labelled')
            return

        self.replot()

    def replot(self):
        self.mplcanvas.fig.clear()
        self.mplcanvas.ax.clear()
        colors = {int(l.idx) : l.color for l in self.LBLM.labels(self.computed_level)}

        selected = []
        selected_title = []
        for d, v in self.options.items():
            if v[1].isChecked():
                selected += [d]
                selected_title += [v[0]]

        N = len(selected)
        axes = np.empty((N, N), dtype=object)

        with sns.axes_style('ticks'):
            for i in range(N):
                ds_i = selected[i]
                data_i = self.DM.load_ds('objects/{}'.format(ds_i))
                for j in range(N-1, -1, -1):
                    ds_j = selected[j]
                    data_j = self.DM.load_ds('objects/{}'.format(ds_j))
                    sharex = axes[-1, j]
                    sharey = None
                    if i != j:
                        sharey = axes[i, 0] if i > 0 else axes[i, 1]

                    axes[i, j] = ax = self.mplcanvas.fig.add_subplot(N, N, i*N+j+1,
                                                                     sharey=sharey,
                                                                     sharex=sharex)
                    if i == j:
                        for l in np.unique(self.labels):
                            sns.kdeplot(data_i[self.labels == l], shade=True, ax=ax,
                                        color=colors[l-1], kernel=self.kernel)
                        ax.yaxis.set_ticklabels([])
                    else:
                        c = [colors[int(l-1)] for l in self.labels]
                        ax.scatter(data_j, data_i, c=c, linewidths=1,
                                   edgecolor="w", s=40)

                    if i < N - 1:
                        ax.set_xticklabels(ax.get_xticklabels(), visible=False)

            for i in range(N):
                axes[-1, i].set_xlabel(selected_title[i])
                axes[i, 0].set_ylabel(selected_title[i])


            sns.despine(self.mplcanvas.fig)
            self.mplcanvas.redraw()

    def on_export_plot(self):
        full_path = QtWidgets.QFileDialog.getSaveFileName(self, "Select output filename",
                                                      filter='*.png')
        if full_path is not None and len(full_path) > 0:
            if not full_path.endswith('.png'):
                full_path += '.png'
        self.mplcanvas.fig.savefig(full_path)

    def on_export_stats(self):
        features = FEATURE_TYPES

        total = len(features)
        nobj = self.num_objects
        data = np.zeros((nobj, total), np.float32)

        for n, ftype in enumerate(features):
            feature = self.DM.load_ds('objects/{}'.format(ftype))
            data[:, n] = feature

        full_path = QtWidgets.QFileDialog.getSaveFileName(self, "Select output filename",
                                                      filter='*.csv')
        if full_path is not None and len(full_path) > 0:
            if not full_path.endswith('.csv'):
                full_path += '.csv'
            df = pd.DataFrame(data, columns=features)
            if self.labels is not None and len(self.labels) > 0:
                df['class'] = [self.LBLM.get(self.computed_level, l-1).name for l in self.labels]
            df.to_csv(full_path)

    def on_kernel_changed(self):
        self.kernel = self.kernels.currentText()
        self.replot()

    def reset(self):
        for k, v in iteritems(self.options):
            if type(k) == tuple:
                v[1].setParent(None)
                del self.options[k]

        self.objects = None
        self.selected_label = None
        self.feature = None
        self.colors = None
        self.color_idx = None
        self.labels = None
        self.computed_level = None
        self.mplcanvas.ax.clear()
        self.mplcanvas.redraw()




import numpy as np
from ..qt_compat import QtGui, QtCore, QtWidgets

from .base import Plugin
from .contrast import Contrast
from .layers import Layers
from ..widgets import HWidgets, LCheckBox, HSize3D, BLabel, PLineEdit, HeaderLabel
from ..core import DataModel, LayerManager, Launcher
from .. import actions as ac


class LayersMenu(QtWidgets.QMenu):

    def __init__(self, parent=None):
        super(LayersMenu, self).__init__(parent=parent)
        self.setStyleSheet('QMenu {'
                           '    border: 1px solid #ccc;'
                           '    background-color: #fbfbfb;'
                           '    border-radius: 4px;'
                           '    min-width: 400px;'
                           '}')
        layout = QtWidgets.QVBoxLayout(self)
        self.layers = Layers()
        layout.addWidget(self.layers)

    def sizeHint(self):
        return self.layers.sizeHint()

class ContrastMenu(QtWidgets.QMenu):

    def __init__(self, parent=None):
        super(ContrastMenu, self).__init__(parent=parent)
        self.setStyleSheet('QMenu {'
                           '    border: 1px solid #ccc;'
                           '    background-color: #fbfbfb;'
                           '    border-radius: 4px;'
                           '    min-width: 400px;'
                           '}')
        layout = QtWidgets.QVBoxLayout(self)
        self.contrast = Contrast()
        layout.addWidget(self.contrast)

    def sizeHint(self):
        return self.contrast.sizeHint()

class Visualization(Plugin):

    name = 'Visualization'

    def __init__(self, parent=None):
        super(Visualization, self).__init__(ptype=Plugin.Plugin, parent=parent)

        self.DM = DataModel.instance()
        self.LM = LayerManager.instance()

        contrastLabel = HeaderLabel('Contrast')
        layersLabel = HeaderLabel('Layers')

        self.addWidget(contrastLabel)
        self.addWidget(Contrast())
        self.addWidget(layersLabel)
        self.addWidget(Layers())





from .base import HSlider, HSize3D, HWidgets, BLabel, PLineEdit, CSlider, HLayer, \
                  RLabel, RCheckBox, LCheckBox, ULabel, MHBoxLayout, MVBoxLayout, \
                  HeaderButton, HEditLabel, ColorButton, FLineEdit, TComboBox, \
                  HLabelParent, PicButton, HeaderLabel, SubHeaderLabel, \
                  RoundedWidget, ComboDialog, SComboBox, CheckableLabels, \
                  SourceCombo, MultiSourceCombo, ActionButton, FileWidget,\
                  SectionCombo

from .mainwindow import MainWindow

from .slice_viewer import SliceViewer
from .conficence_viewer import ConfidenceViewer
from .mpl_widgets import OrthogonalViewer
from .compare_levels import QuantitativeAnalysis





import numpy as np
from ..qt_compat import QtGui, QtCore, QtWidgets

from ..widgets import HSlider, HWidgets, ActionButton, TComboBox
from ..widgets.mpl_widgets import MplCanvas
from .base import Plugin
from ..core import DataModel, LayerManager, Launcher, LabelManager
from .. import actions as ac

import logging as log
import seaborn as sns
from matplotlib import pyplot as plt

class Contrast(QtWidgets.QWidget):

    name = 'Contrast'

    def __init__(self, parent=None):
        super(Contrast, self).__init__(parent=parent)

        vbox = QtWidgets.QVBoxLayout(self)
        self.setLayout(vbox)

        self.DM = DataModel.instance()
        self.LM = LayerManager.instance()
        self.LBLM = LabelManager.instance()
        self.launcher = Launcher.instance()

        self.sld_vmin = HSlider('VMin', 0, 1, 0)
        self.sld_vmax = HSlider('VMax', 0, 1, 1)

        vbox.addWidget(self.sld_vmin)
        vbox.addWidget(self.sld_vmax)

        visualize = QtWidgets.QCheckBox('View Histogram')
        default = ActionButton('Default')
        default.clicked.connect(self.restore_default)
        vbox.addWidget(HWidgets(visualize, None, default, stretch=[0,1,0]))

        self.visualization = MplCanvas(autoscale=True, axisoff=True)
        self.visualization.setFixedWidth(293)
        self.visualization.setFixedHeight(200)
        self.visualization.setVisible(False)
        visualize.toggled.connect(self.on_visualize)
        vbox.addWidget(HWidgets(None, self.visualization, None, stretch=[1,0,1]))

        # Slots
        self.DM.vmin_changed.connect(self.update_vmin)
        self.DM.vmax_changed.connect(self.update_vmax)
        self.DM.evmin_changed.connect(self.update_evmin)
        self.DM.evmax_changed.connect(self.update_evmax)

        self.sld_vmin.valueChanged.connect(self.evmin_changed)
        self.sld_vmax.valueChanged.connect(self.evmax_changed)

        self.DM.select_channel.connect(self.view_channel)
        self.DM.update_channel.connect(self.view_channel)

        self.hist_computed = False
        self.current_channel = '/data'
        self.vmin = self.vmax = self.evmin = self.evmax = 0
        self.update_contrast(self.current_channel)

        self.threshold = QtWidgets.QCheckBox('Threshold')
        self.cmb_save_to = TComboBox('Save to:', [])
        self.btn_save_to = QtWidgets.QPushButton('Save')
        vbox.addWidget(HWidgets(self.threshold, self.cmb_save_to, self.btn_save_to,
                                stretch=[1]))

        self.threshold.toggled.connect(self.on_threshold)
        self.btn_save_to.clicked.connect(self.save_threshold)

        self.LBLM.levelAdded.connect(self.on_levels_changed)
        self.LBLM.levelLoaded.connect(self.on_levels_changed)
        self.LBLM.levelRemoved.connect(self.on_levels_changed)

        self.LBLM.labelAdded.connect(self.on_levels_changed)
        self.LBLM.labelLoaded.connect(self.on_levels_changed)
        self.LBLM.labelNameChanged.connect(self.on_levels_changed)
        self.LBLM.labelRemoved.connect(self.on_levels_changed)

        self.all_labels = []

    def on_threshold(self, flag):
        data = self.LM.get('Data', 'Data')
        data.binarize = flag
        self.LM.update()

    def save_threshold(self):
        if len(self.all_labels) == 0 or not self.threshold.isChecked():
            return
        data = self.LM.get('Data', 'Data')
        dataset = data.data
        vmin = data.vmin
        vmax = data.vmax
        idx = self.cmb_save_to.currentIndex()
        level = self.LBLM.dataset(self.all_labels[idx][0])
        label = self.all_labels[idx][1].idx
        Launcher.instance().run(ac.save_threshold, caption='Saving threshold',
                                source=dataset, vmin=vmin, vmax=vmax, level=level, label=label)

    def on_levels_changed(self):
        levels = self.LBLM.levels()
        self.all_labels = []
        self.cmb_save_to.clear()
        for level in levels:
            labels = self.LBLM.labels(level)
            for label in labels:
                self.all_labels.append((level, label))
                self.cmb_save_to.addItem('Level {}/{}'.format(level, label.name))

    def view_channel(self, channel='/data'):
        layer = self.LM.get('Data', 'Data')
        layer.data = channel
        vmin, vmax = self.update_contrast(channel)
        layer.vmin = vmin
        layer.vmax = vmax
        self.LM.update()
        if self.visualization.isVisible():
            self.compute_hist()
        else:
            self.hist_computed = False

    def restore_default(self):
        attrs = self.DM.attrs(self.current_channel)
        evmin = attrs['default_evmin']
        evmax = attrs['default_evmax']
        self.evmin_changed(evmin)
        self.evmax_changed(evmax)
        self.LM.update()

    def update_contrast(self, channel):
        self.current_channel = channel

        attrs = self.DM.attrs(channel)
        vmin = attrs['vmin']
        vmax = attrs['vmax']
        evmin = attrs['evmin']
        evmax = attrs['evmax']

        self.update_vmin(vmin)
        self.update_vmax(vmax)
        self.update_evmin(evmin)
        self.update_evmax(evmax)

        return evmin, evmax

    def evmin_changed(self, val):
        if val >= self.evmax:
            val = self.evmax - 1e-10
        self.DM.set_attrs(self.current_channel, dict(evmin=val))
        self.DM.evmin_changed.emit(val)
        self.LM.setVMin('Data', 'Data', val)

    def evmax_changed(self, val):
        if val <= self.evmin:
            val = self.evmin +  1e-10
        self.DM.set_attrs(self.current_channel, dict(evmax=val))
        self.DM.evmax_changed.emit(val)
        self.LM.setVMax('Data', 'Data', val)

    def update_vmin(self, val):
        self.vmin = val
        self.sld_vmin.setMinimum(val)
        self.sld_vmax.setMinimum(val)

    def update_vmax(self, val):
        self.vmax = val
        self.sld_vmin.setMaximum(val)
        self.sld_vmax.setMaximum(val)

    def update_evmin(self, val):
        self.evmin = val
        if self.hist_computed:
            aux = ([val, val], [0, 1])
            self.minline.set_data(aux)
            aux = self.minspan.get_xy(); aux[2:4, 0] = val
            self.minspan.set_xy(aux)
            self.visualization.redraw()
        self.sld_vmin.blockSignals(True)
        self.sld_vmin.setValue(val)
        self.sld_vmin.blockSignals(False)

    def update_evmax(self, val):
        self.evmax = val
        if self.hist_computed:
            aux = ([val, val], [0, 1])
            self.maxline.set_data(aux)
            aux = self.maxspan.get_xy(); aux[[0,1,4], 0] = val
            self.maxspan.set_xy(aux)
            self.visualization.redraw()
        self.sld_vmax.blockSignals(True)
        self.sld_vmax.setValue(val)
        self.sld_vmax.blockSignals(False)

    def on_visualize(self, bol):
        if bol and not self.hist_computed:
            self.compute_hist()
        self.visualization.setVisible(bol)

    def compute_hist(self):
        self.launcher.setup('Computing histogram')
        x, y = self.DM.get_histogram(self.current_channel)

        log.info('+ Plotting')
        ax = self.visualization.ax
        ax.clear()

        y = np.maximum(0, y)
        width = 0.7 * (y[1] - y[0])
        ax.plot(x, y)
        ax.fill_betweenx(y, 1e-12, x, alpha=0.25)

        self.minspan = ax.axvspan(self.vmin, self.evmin, color='r', alpha=0.2)
        self.minline = ax.axvline(self.evmin, linewidth=1, color='r')

        self.maxspan = ax.axvspan(self.evmax, self.vmax, color='r', alpha=0.2)
        self.maxline = ax.axvline(self.evmax, linewidth=1, color='r')

        ax.set_xlim([self.vmin, self.vmax])
        ax.set_xticks([])
        ax.set_yticks([])

        self.visualization.redraw()

        self.hist_computed = True
        self.launcher.cleanup()



#!/usr/bin/env python


import os
from sys import platform
from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration, get_numpy_include_dirs

import urllib
try:
    import urllib.request
except:
    pass

import shutil, glob
import zipfile

from survos.build import custom_build_ext, locate_cuda

base_path = os.path.abspath(os.path.dirname(__file__))
source_path = os.path.join(base_path, 'qpbo_src')

def get_qpbo():
    if os.path.isdir(source_path):
        return
    else:
        os.mkdir(source_path)

    if hasattr(urllib, "urlretrieve"):
        urlretrieve = urllib.urlretrieve
    else:
        urlretrieve = urllib.request.urlretrieve

    qpbo_version = 'QPBO-v1.4.src'
    qpbo_file = '{}.zip'.format(qpbo_version)

    urlretrieve('http://pub.ist.ac.at/~vnk/software/{}'.format(qpbo_file), qpbo_file)
    with zipfile.ZipFile(qpbo_file) as zf:
        zf.extractall(source_path)

    for f in glob.glob(os.path.join(source_path, qpbo_version, '*')):
        shutil.move(f, os.path.dirname(os.path.dirname(f)))

    os.rmdir(os.path.join(source_path, qpbo_version))
    os.remove(qpbo_file)

def cuda_extension(config, name, sources, CUDA, gcc_args=[], gpp_args=[]):
    if platform == "win32":
        config.add_extension(name, sources=sources,
                         library_dirs=[CUDA['lib64']],
                         libraries=['cudart'], language='c++',
                         include_dirs = [get_numpy_include_dirs()[0],
                                         CUDA['include'], 'src'])
    else:
        config.add_extension(name, sources=sources,
                         library_dirs=[CUDA['lib64']],
                         libraries=['cudart', 'stdc++'], language='c++',
                         runtime_library_dirs=[CUDA['lib64']],
                         extra_compile_args={
                            'gcc': gcc_args,
                            'g++': gpp_args,
                            'nvcc': ['-arch=sm_30',
                                     '-gencode=arch=compute_20,code=sm_20',
                                     '-gencode arch=compute_30,code=sm_30',
                                     '-gencode arch=compute_35,code=sm_35',
                                     '-gencode arch=compute_37,code=sm_37',
                                     '-gencode arch=compute_50,code=sm_50',
                                     '-gencode arch=compute_52,code=sm_52',
                                     '-gencode arch=compute_52,code=compute_52',
                                     '-gencode=arch=compute_60,code=sm_60',
                                     '-gencode=arch=compute_61,code=sm_61',
                                     '--ptxas-options=-v', '-c',
                                     '--compiler-options', "'-fPIC'"]
                         },
                         include_dirs = [get_numpy_include_dirs()[0],
                                         CUDA['include'], 'src'])


def configuration(parent_package='', top_path=None):
    config = Configuration('lib', parent_package, top_path,
                           cmdclass={'build_ext': custom_build_ext})

    CUDA = locate_cuda()
    extra_libraries = []
    if platform != "win32":
        extra_libraries = ["stdc++"]
    files = ['_supersegments.pyx']
    config.add_extension('_supersegments', sources=files,
                         language='c++', libraries=extra_libraries,
                         include_dirs=[get_numpy_include_dirs()])

    sources = ['src/cuda.cu', 'src/tv.cu', 'src/diffusion.cu',
               'src/chambolle2005.cu', 'src/chambolle2011.cu',
               'src/bregman.cu',  '_preprocess.pyx']
    cuda_extension(config, '_preprocess', sources, CUDA)

    sources = ['src/slic.cu', 'src/cuda.cu', '_superpixels.pyx']
    cuda_extension(config, '_superpixels', sources, CUDA)

    sources = ['src/convolutions_raw.cu',
               'src/convolutions_separable.cu',
               'src/convolutions_separable_shared.cu',
               'src/cuda.cu', '_convolutions.pyx']
    cuda_extension(config, '_convolutions', sources, CUDA)

    sources = ['src/symmetric_eigvals3S.cu', 'src/cuda.cu', '_channels.pyx']
    cuda_extension(config, '_channels', sources, CUDA)

    config.add_extension('_spencoding', sources=['_spencoding.pyx'],
                         include_dirs=get_numpy_include_dirs())
    config.add_extension('_features', sources=['_features.pyx'],
                         include_dirs=get_numpy_include_dirs())
    config.add_extension('_rag', sources=['_rag.pyx'],
                         include_dirs=get_numpy_include_dirs())

    config.add_extension('_dist', sources=['_dist.pyx'],
                         include_dirs=get_numpy_include_dirs())

    get_qpbo()
    qpbo_directory = source_path
    files = ["QPBO.cpp", "QPBO_extra.cpp", "QPBO_maxflow.cpp",
             "QPBO_postprocessing.cpp"]
    files = [os.path.join(qpbo_directory, f) for f in files]
    files = ['_qpbo.pyx'] + files
    config.add_extension('_qpbo', sources=files, language='c++',
                         libraries=extra_libraries,
                         include_dirs=[qpbo_directory, get_numpy_include_dirs()],
                         library_dirs=[qpbo_directory],
                         extra_compile_args=[],
                         extra_link_args=[])

    return config


if __name__ == '__main__':
    config = configuration(top_path='').todict()
    setup(**config)





import numpy as np

from sklearn.utils import shuffle
from sklearn.decomposition import IncrementalPCA, PCA, \
                                  DictionaryLearning, MiniBatchDictionaryLearning
from sklearn.cluster import MiniBatchKMeans, KMeans


class Quantizer(object):

    def __init__(self, n_bins=100, n_samples=10000, random_state=None,
                 incremental=False, n_jobs=1, **kwargs):
        self.incremental = incremental
        if self.incremental:
            self.kmeans = MiniBatchKMeans(n_clusters=n_bins,
                                          random_state=random_state, **kwargs)
        else:
            self.kmeans = KMeans(n_clusters=n_bins, random_state=random_state,
                                 n_jobs=n_jobs, **kwargs)
        self.n_samples = n_samples
        self.random_state = random_state
        self.incremental

    def fit(self, X, y=None):
        if self.n_samples is not None and self.n_samples < X.shape[0]:
            X = shuffle(X, random_state=self.random_state)[:self.n_samples]
        self.kmeans.fit(X)

    def partial_fit(self, X, y=None):
        if not self.incremental:
            raise Exception('`partial_fit` not supported if not `incremental`')
        self.kmeans.partial_fit(X)

    def transform(self, X):
        y = self.predict(X)
        return self.kmeans.cluster_centers_[y]

    def fit_transform(self, X):
        self.fit(X)
        return self.transform(X)

    def predict(self, X):
        return self.kmeans.predict(X)

    def fit_predict(self, X):
        self.fit(X)
        return self.predict(X)


class Textonizer(object):

    def __init__(self, n_textons=100, n_samples=10000, whiten=True,
                 random_state=None, incremental=False, n_jobs=1, **kwargs):
        self.incremental = incremental
        if self.incremental:
            self.kmeans = MiniBatchKMeans(n_clusters=n_textons,
                                          random_state=random_state, **kwargs)
            self.pca = IncrementalPCA(whiten=whiten)
        else:
            self.kmeans = KMeans(n_clusters=n_textons, random_state=random_state,
                                 n_jobs=n_jobs, **kwargs)
            self.pca = PCA(whiten=whiten)
        self.n_samples = n_samples
        self.random_state = random_state
        self.incremental = incremental

    def fit(self, X, y=None):
        if self.n_samples is not None and self.n_samples < X.shape[0]:
            X = shuffle(X, random_state=self.random_state)[:self.n_samples]
        X = self.pca.fit_transform(X)
        self.kmeans.fit(X)

    def partial_fit(self, X, y=None):
        if not self.incremental:
            raise Exception('`partial_fit` not supported if not `incremental`')
        self.pca.partial_fit(X)
        X = self.pca.transform(X)
        self.kmeans.partial_fit(X)

    def transform(self, X):
        y = self.predict(X)
        return self.kmeans.cluster_centers_[y]

    def fit_transform(self, X, y=None):
        self.fit(X, y)
        return self.transform(X)

    def predict(self, X):
        X = self.pca.transform(X)
        return self.kmeans.predict(X)

    def fit_predict(self, X):
        self.fit(X)
        return self.predict(X)




import numpy as np
from ..qt_compat import QtGui, QtCore, QtWidgets

import logging as log

from matplotlib.colors import ListedColormap

from ..core import Launcher
from ..widgets import HSize3D, HWidgets, PLineEdit, SourceCombo, ActionButton
from .base import Plugin
from ..core import DataModel, LayerManager
from .. import actions as ac

from ..lib._features import find_boundaries

class SuperVoxels(QtWidgets.QWidget):

    name = 'SuperVoxels'

    def __init__(self, parent=None):
        super(SuperVoxels, self).__init__(parent=parent)

        self.DM = DataModel.instance()
        self.LM = LayerManager.instance()
        self.setLayout(QtWidgets.QVBoxLayout())

        self.source_combo = SourceCombo()
        self.layout().addWidget(HWidgets(None, 'Source:', self.source_combo, stretch=[1]))

        self.sp_size = HSize3D('SP shape', (10, 10, 10), txtwidth=70)
        self.sp_spacing = HSize3D('Spacing', (1, 1, 1), txtwidth=70)

        self.layout().addWidget(self.sp_size)
        self.layout().addWidget(self.sp_spacing)

        self.txt_comp = PLineEdit(20., parse=float)

        dummy = HWidgets('Compactness:', self.txt_comp, \
                         stretch=[0,0,0,0])
        self.layout().addWidget(dummy)


        self.btn_apply = ActionButton('Apply')
        dummy = HWidgets(QtWidgets.QWidget(), self.btn_apply,\
                         stretch=[1,0])

        self.layout().addWidget(dummy)

        self.btn_apply.clicked.connect(self.apply_supervoxels)

    def apply_supervoxels(self):
        target = self.source_combo.value()

        spshape = self.sp_size.value()

        self.params = {
            'sp_shape' : spshape,
            'spacing' : self.sp_spacing.value(),
            'compactness' : self.txt_comp.value()
        }

        in_data = target

        out_sv = 'supervoxels/supervoxels'
        out_svindex = 'supervoxels/supervoxels_idx'
        out_svtable = 'supervoxels/supervoxels_table'
        out_svedges = 'supervoxels/graph_edges'
        out_svweights = 'supervoxels/graph_edge_weights'

        self.DM.remove_dataset(out_sv)
        self.DM.remove_dataset(out_svindex)
        self.DM.remove_dataset(out_svtable)
        self.DM.remove_dataset(out_svedges)
        self.DM.remove_dataset(out_svweights)

        Launcher.instance().run(ac.create_supervoxels,
                                dataset=in_data,
                                out_sv=out_sv,
                                out_svindex=out_svindex, out_svtable=out_svtable,
                                out_svedges=out_svedges, out_svweights=out_svweights,
                                cb=self.on_supervoxels,
                                caption='Creating SuperVoxels..',
                                **self.params)

    def on_supervoxels(self, params):
        svlabels, svtotal, sortindex, sorttable, edges, weights = params
        self.update_supervoxel_layer(svlabels, sortindex, sorttable, svtotal, visible=True)

    def update_supervoxel_layer(self, svlabels, sortindex, sorttable, total_sv, visible=False):
        if svlabels is not None:
            self.DM.svlabels = svlabels
            self.DM.svtotal = total_sv
            self.DM.svindex = sortindex
            self.DM.svtable = sorttable

        if self.LM.isin('SuperVoxels', 'Super-Regions'):
            self.LM.get('SuperVoxels', 'Super-Regions').data = self.DM.svlabels
            self.LM.setVisible('SuperVoxels', 'Super-Regions', True)
        else:
            cmap = ListedColormap(['MidnightBlue'])
            self.LM.addLayer(self.DM.svlabels, 'SuperVoxels', level='Super-Regions',
                             pre=find_boundaries, cmap=cmap, background=0,
                             vmin=0.1, vmax=1, visible=visible)
        self.LM.update()

    def update_params(self, source, total_sv, sv_shape, svacing, compactness):
        log.info('  * {} supervoxels loaded'.format(total_sv))
        self.source_combo.loadSource(source)
        self.sp_size.setValue(sv_shape)
        self.sp_spacing.setValue(svacing)
        self.txt_comp.setText(str(compactness))




import numpy as np
from ..qt_compat import QtGui, QtCore

from .base import Plugin
from .supervoxels import SuperVoxels
from .megavoxels import MegaVoxels
from ..widgets import HWidgets, LCheckBox, HSize3D, BLabel, PLineEdit, HeaderLabel
from ..core import DataModel, LayerManager, Launcher
from .. import actions as ac


class SuperRegions(Plugin):

    name = 'Super Regions'

    def __init__(self):
        super(SuperRegions, self).__init__(ptype=Plugin.Plugin)

        self.DM = DataModel.instance()
        self.LM = LayerManager.instance()

        svLabel = HeaderLabel('SuperVoxels')
        mvLabel = HeaderLabel('MegaVoxels')

        self.supervoxels = SuperVoxels()
        self.megavoxels = MegaVoxels()

        self.addWidget(svLabel)
        self.addWidget(self.supervoxels)
        self.addWidget(mvLabel)
        self.addWidget(self.megavoxels)

    def load_supervoxels(self, data, data_idx, data_table,
                         source, sv_shape, spacing, compactness, svtotal):
        self.supervoxels.update_supervoxel_layer(data, data_idx, data_table, svtotal)
        self.supervoxels.update_params(source, svtotal, sv_shape, spacing, compactness)

    def load_megavoxels(self, data, data_idx, data_table,
                        source, lamda, nbins, gamma, mvtotal):
        self.megavoxels.update_megavoxel_layer(data, data_idx, data_table, mvtotal)
        self.megavoxels.update_params(source, mvtotal, lamda, nbins, gamma)



#!/usr/bin/env python


import os
from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration, get_numpy_include_dirs

from survos.build import custom_build_ext

base_path = os.path.abspath(os.path.dirname(__file__))


def configuration(parent_package='', top_path=None):
    config = Configuration('survos', parent_package, top_path,
                           cmdclass={'build_ext': custom_build_ext})
    config.add_subpackage('lib')
    return config


if __name__ == '__main__':
    config = configuration(top_path='').todict()
    setup(**config)



import matplotlib as mpl
mpl.use("Qt5Agg", force=True)

from ..qt_compat import QtGui, QtCore, QtWidgets

import matplotlib.pyplot as plt

import numpy as np

from .base import WSliderPane, SComboBox, ULabel

from ..core import DataModel, LayerManager, LabelManager, Launcher
from ..plugins.base import Plugin

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5 import FigureManagerQT
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg


Axial = DataModel.instance().Axial
Sagittal = DataModel.instance().Sagittal
Coronal = DataModel.instance().Coronal

views = [Axial, Sagittal, Coronal]


class OrthogonalViewer(Plugin):

    name = 'Orthogonal Viewer'

    def __init__(self, ptype=Plugin.Widget):
        super(OrthogonalViewer, self).__init__(ptype=ptype)

        for i, orient in enumerate(views):
            mc = MplCanvas(orient=orient)
            self.layout.addWidget(mc, i%2, i//2)

class MplCanvas(QtWidgets.QWidget):

    def __init__(self, orient=Axial, axisoff=True, autoscale=False, **kwargs):
        super(MplCanvas, self).__init__()

        self.orient = orient
        self.setLayout(QtWidgets.QVBoxLayout())

        # Figure
        self.fig, self.ax, self.canvas = self.figimage(axisoff=axisoff)
        self.pressed = False
        self.ax.autoscale(enable=autoscale)
        self.layout().addWidget(self.canvas, 1)
        self.canvas.mpl_connect('button_press_event',self.on_press)

    def replot(self):
        self.ax.clear()
        self.ax.cla()
        self.ax.spines['left'].set_position(('outward', 3))
        self.ax.spines['bottom'].set_position(('outward', 3))
        self.ax.spines['right'].set_position(('outward', 3))
        self.ax.spines['top'].set_position(('outward', 3))

    def redraw(self):
        self.canvas.draw_idle()

    def figimage(self, scale=1, dpi=None, axisoff=True):
        fig = plt.figure(figsize=(10, 10))
        canvas = FigureCanvas(fig)
        ax = fig.add_subplot(111)
        if axisoff:
            fig.subplots_adjust(left=0.03, bottom=0.05, right=0.97, top=0.99)
        canvas.draw()
        return fig, ax, canvas

    def on_press(self, event):
        self.setFocus()

class PerspectiveCanvas(MplCanvas):

    def __init__(self, orient=Axial, axisoff=True, autoscale=False, **kwargs):
        super(PerspectiveCanvas, self).__init__(axisoff=axisoff,
                                                autoscale=autoscale)
        self.DM = DataModel.instance()
        self.LM = LayerManager.instance()
        self.LBLM = LabelManager.instance()
        self.launcher = Launcher.instance()

        self.idx = self.DM.data_shape[orient] // 2

        topbox = QtWidgets.QHBoxLayout()
        self.layout().insertLayout(0, topbox)

        # Slider
        self.slider = WSliderPane(self.DM.data_shape[self.orient]-1, orient=1)
        self.slider.setTracking(True)
        topbox.addWidget(self.slider, 1)

        # Slider Text
        self.text_idx = ULabel(str(self.idx))
        topbox.addWidget(self.text_idx)

        # Slots
        self.DM.cropped.connect(self.update_slides)

    def update_slides(self):
        self.slider.setMaximum(self.DM.data_shape[self.orient]-1)
        self.slider.setValue(self.DM.data_shape[self.orient]//2)

    def on_perspective_changed(self, orient):
        self.orient = orient

        for layer in self.LM.layers():
            layer.orient = orient

        self.blockSignals(True)
        self.slider.setMaximum(self.DM.data_shape[self.orient]-1)
        self.idx = (self.DM.data_shape[self.orient]) // 2
        self.slider.setValue(self.idx)
        self.blockSignals(False)
        self.text_idx.setText(str(self.idx))

        self.replot()

    def on_source_changed(self, idx):
        pass

class ConfidenceCanvas(MplCanvas):

    def __init__(self, k=None, name=None, *args, **kwargs):
        super(ConfidenceCanvas, self).__init__(*args, **kwargs)
        self.name = name
        self.data = self.DM.splabels
        self.k = k
        self.replot()

    def replot(self):
        self.ax.clear()
        slice = self.get_slice(self.idx)
        self.ax.imshow(slice, 'optiona', vmin=0, vmax=1)
        self.ax.set_title('P(%s)' % self.name)
        self.ax.set_xticks([])
        self.ax.set_yticks([])
        self.redraw()

    def get_slice(self, idx):
        if self.k is None:
            shape = list(self.DM.data_shape)
            del shape[self.orient]
            data = np.zeros(shape, int)
            return data if self.orient < 2 else data.T

        if self.orient == Axial:
            data = self.data[idx, :, :]
        elif self.orient == Sagittal:
            data = self.data[:, idx, :]
        else:
            data = self.data[:, :, idx].T

        return self.DM.spprobabilities[:, self.k][data]

    def update_volume(self, idx=None):
        if idx is not None:
            self.idx = idx
        slide = self.get_slice(self.idx)
        self.ax.images[0].set_array(slide)
        self.redraw()


class FigureCanvas(FigureCanvasQTAgg):
    def __init__(self, figure, **kwargs):
        self.fig = figure
        super(FigureCanvas, self).__init__(self.fig)
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
                            QtWidgets.QSizePolicy.Expanding)
        self.updateGeometry()
        self.mouse_lines = None
        self.mouse_color = QtCore.Qt.black
        self.mouse_width = 1

    def resizeEvent(self, event):
        super(FigureCanvas, self).resizeEvent(event)
        self.resize_event()

    def setColor(self, color):
        self.mouse_color = color

    def setMouseWidth(self, width):
        self.mouse_width = width

    def updateMouseLines(self, y, x):
        y = self.height() - int(y)
        x = int(x)
        if self.mouse_lines is None:
            self.mouse_lines = [[y], [x]]
        else:
            self.mouse_lines[0].append(y)
            self.mouse_lines[1].append(x)
        self.update()

    def setMouseLines(self, lines):
        self.mouse_lines = lines
        self.update()

    def paintEvent(self, event):
        super(FigureCanvas, self).paintEvent(event)

        if self.mouse_lines is None:
            return

        qp = QtGui.QPainter()
        qp.begin(self)

        qp.setPen(QtGui.QPen(self.mouse_color, self.mouse_width))

        for i in range(len(self.mouse_lines[0]) - 1):
            y0, x0 = self.mouse_lines[0][i], self.mouse_lines[1][i]
            y1, x1 = self.mouse_lines[0][i+1], self.mouse_lines[1][i+1]
            qp.drawLine(x0, y0, x1, y1)

        qp.end()





import numpy as np
import pandas as pd
from ..qt_compat import QtGui, QtCore, QtWidgets

from .mpl_widgets import PerspectiveCanvas, MplCanvas
from .base import HWidgets, TComboBox, RoundedWidget, ColorButton, PLineEdit, \
                  SComboBox, CheckableLabels, SourceCombo

from .. import actions as ac

from ..plugins.base import Plugin
from ..core import DataModel, LabelManager, Launcher, LayerManager

import seaborn as sns
import logging as log


FEATURE_OPTIONS = [
    'Average Intensity',
    #'Median Intensity',
    'Sum Intensity',
    'Standard Devitation',
    'Variance',
    'Size (Area)',
    'Log10(Size) (Area)',
    'Size (Bounding Box)',
    'Depth (Bounding Box)',
    'Height (Bounding Box)',
    'Width (Bounding Box)',
    'Log10(Size) (Bounding Box)',
    'Size (Oriented Bounding Box)',
    '1st Axis (Oriented Bounding Box)',
    '2nd Axis (Oriented Bounding Box)',
    '3rd Axis (Oriented Bounding Box)',
    'Log10(Size) (Oriented Bounding Box)',
    'Position (X)',
    'Position (Y)',
    'Position (Z)'
]

FEATURE_TYPES = [
    'intensity',
    #'median',
    'sum',
    'std',
    'var',
    'area',
    'log_area',
    'size_bbox',
    'depth_bbox',
    'height_bbox',
    'width_bbox',
    'log_size_bbox',
    'size_ori_bbox',
    'depth_ori_bbox',
    'height_ori_bbox',
    'width_ori_bbox',
    'log_size_ori_bbox',
    'z_pos',
    'y_pos',
    'x_pos'
]

class LabelCanvas(PerspectiveCanvas):

    labelSelected = QtCore.pyqtSignal(int)

    def __init__(self):
        super(LabelCanvas, self).__init__(axisoff=False, autoscale=False)

        self.DM = DataModel.instance()
        self.DM.evmin_changed.connect(self.evmin_changed)
        self.DM.evmax_changed.connect(self.evmax_changed)
        self.DM.roi_changed.connect(self.initialize)
        self.slider.valueChanged.connect(self.on_value_changed)
        self.canvas.mpl_connect('button_release_event', self.on_select_object)
        self.fig.subplots_adjust(left=0.05, bottom=0.05, right=0.95, top=0.95)

        self.data = None
        self.objects = None
        self.num_objects = None
        self.selected_object = None
        self.labels = None
        self.color_idx = None
        self.colors = None
        self.initialize()

    def initialize(self):
        self.data = '/data'
        if self.data is None:
            return False
        self.shape = self.DM.region_shape()
        self.vmin = self.DM.attr('/data', 'evmin')
        self.vmax = self.DM.attr('/data', 'evmax')
        self.slider.setMaximum(self.shape[0]-1)
        self.slider.setValue(self.shape[0]//2)
        self.on_value_changed(self.shape[0]//2)
        return True

    def on_value_changed(self, val):
        self.idx = val
        self.text_idx.setText(str(val))
        self.replot()

    def evmin_changed(self, vmin):
        self.vmin = vmin
        if len(self.ax.images):
            self.ax.images[0].set_clim(vmin=vmin)
        self.redraw()

    def evmax_changed(self, vmax):
        self.vmax = vmax
        if len(self.ax.images):
            self.ax.images[0].set_clim(vmax=vmax)
        self.redraw()

    def replot(self):
        self.ax.clear()
        if self.data is None:
            return
        curr_data = self.DM.load_slices(self.data, self.idx)[0]
        self.ax.imshow(curr_data, 'gray',
                       vmin=self.vmin, vmax=self.vmax)
        if self.objects:
            objs = self.DM.load_slices(self.objects, self.idx)[0]
            omax = objs.max()
            if omax > 0:
                self.ax.contour(objs, colors='#0000FF', linewidths=2,
                                levels=range(omax))

        if self.labels is not None:
            objs = self.DM.load_slices(self.objects, self.idx)[0]
            labels = self.labels[objs]
            for cidx in self.color_idx:
                color = self.colors[cidx]
                curr_obj = (labels == cidx)
                if np.unique(curr_obj).size > 1:
                    self.ax.contour(curr_obj, colors=color, linewidths=2)

        if self.selected_object is not None:
            objs = self.DM.load_slices(self.objects, self.idx)[0]
            mask = (objs == self.selected_object)
            if mask.any():
                self.ax.contour(mask, colors='#FF0000',
                                linewidths=2)
        self.ax.axis('image')
        self.redraw()

    def set_objects(self, objects, num_objects):
        self.objects = objects
        self.num_objects = num_objects
        self.selected_object = None
        self.replot()

    def on_select_object(self, ev):
        if ev.inaxes != self.ax or self.objects is None:
            return

        y = int(ev.ydata)
        x = int(ev.xdata)
        l = self.DM.load_slices(self.objects, self.idx)[0, y, x]
        if l > 0:
            self.selected_object = l
            self.replot()
            self.labelSelected.emit(l)

    def select_object(self, obj):
        self.selected_object = obj
        self.replot()

    def colour(self, labels, idx, colors):
        self.labels = np.r_[-1, labels]
        self.color_idx = idx
        self.colors = colors
        self.replot()

    def reset(self):
        self.labels = None
        self.objects = None
        self.num_objects = None
        self.selected_object = None
        self.replot()


class LabelExplorer(QtWidgets.QWidget):

    def __init__(self, label_canvas):
        super(LabelExplorer, self).__init__()

        self.label_canvas = label_canvas
        self.DM = DataModel.instance()
        self.LBLM = LabelManager.instance()

        vbox = QtWidgets.QVBoxLayout()
        self.setLayout(vbox)

        self.level_combo = SComboBox()
        self.level_combo.setMinimumWidth(200)
        self.label_combo = CheckableLabels(-1)
        self.label_combo.setMinimumWidth(200)
        self.source_combo = SourceCombo()
        self.source_combo.setMinimumWidth(200)
        self.label_btn = QtWidgets.QPushButton('Label')
        vbox.addWidget(HWidgets(self.level_combo, self.label_combo,
                                self.source_combo, self.label_btn,
                                stretch=[1, 1, 1, 0]))

        self.mplcanvas = MplCanvas(axisoff=False, autoscale=True)
        self.label_rules = LabelRules()

        self.feature_combo = TComboBox('Select measure:', FEATURE_OPTIONS)
        self.kernels = TComboBox('Fit kernel:', ['gau', 'cos', 'biw', 'epa',
                                                'tri', 'triw'],
                                 selected=2)
        self.export_plot = QtWidgets.QPushButton('Export plot')
        self.export_stats = QtWidgets.QPushButton('Export Stats')
        vbox.addWidget(HWidgets(self.feature_combo, None,
                                self.kernels, self.export_plot, self.export_stats,
                                stretch=[0, 1, 0, 0, 0]))

        splitter = QtWidgets.QSplitter(0)
        vbox.addWidget(splitter)

        splitter.addWidget(self.mplcanvas)
        splitter.addWidget(self.label_rules)

        self.LBLM.levelLoaded.connect(self.on_level_added)
        self.LBLM.levelAdded.connect(self.on_level_added)
        self.LBLM.levelRemoved.connect(self.on_level_removed)

        self.level_combo.currentIndexChanged.connect(self.on_level_changed)
        self.label_btn.clicked.connect(self.on_label_clicked)
        self.feature_combo.currentIndexChanged.connect(self.on_feature_changed)
        self.label_canvas.labelSelected.connect(self.on_label_selected)
        self.export_plot.clicked.connect(self.on_export_plot)
        self.export_stats.clicked.connect(self.on_export_stats)
        self.kernels.currentIndexChanged.connect(self.on_kernel_changed)
        self.label_rules.computed.connect(self.on_rules_computed)
        self.label_rules.compute_others.connect(self.on_rules_computed)

        self.mplcanvas.canvas.mpl_connect('button_release_event',
                                           self.on_select_object)

        self.selected_label = None
        self.kernel = 'biw'
        self.feature = None
        self.colors = None
        self.color_idx = None
        self.labels = None
        self.objects = None
        self.computed_level = None
        self.levels = []

    def on_level_changed(self, index):
        self.label_combo.clear()

        if index < 0:
            return

        self.label_combo.selectLevel(self.levels[index])

    def on_level_added(self, level):
        self.levels.append(level)
        self.level_combo.addItem('Level {}'.format(level))
        if level == 0:
            self.on_level_changed(0)
            self.label_combo.selectLevel(level)

    def on_level_removed(self, level):
        i = self.levels.index(level)
        self.level_combo.removeItem(i)
        del self.levels[i]
        if self.computed_level is not None and level == self.computed_level:
            self.reset()

    def on_label_clicked(self):
        self.reset()

        level = self.levels[self.level_combo.currentIndex()]
        self.computed_level = level
        self.LBLM.save(level)
        labels = self.label_combo.getSelectedLabels()

        in_dset = self.LBLM.dataset(level)
        out_dset = 'objects/objects'
        source = self.source_combo.value()

        out_features = []
        for f in FEATURE_TYPES:
            out_features.append('objects/{}'.format(f))

        self.label_rules.initialize()

        self.DM.remove_dataset('objects/objects')
        self.DM.remove_dataset('objects/objlabels')

        Launcher.instance().run(ac.label_objects, dataset=in_dset, source=source,
                                out=out_dset, out_features=out_features,
                                labels=labels, caption='Labelling objects...',
                                cb=self.on_objects_labelled)

    def on_objects_labelled(self, params):
        objects, num_objects = params[:2]
        if objects is None:
            return
        self.objects = objects
        self.num_objects = num_objects
        self.label_canvas.set_objects(objects, num_objects)
        self.on_feature_changed()

    def on_feature_changed(self):
        feature_idx = self.feature_combo.currentIndex()
        stype = FEATURE_TYPES[feature_idx]
        self.show_stats(stype=stype)

    def show_stats(self, stype='intensity'):
        if stype == 'intensity':
            self.title = 'Average Intensity inside objects'
        elif stype == 'median':
            self.title = 'Median Intensity inside objects'
        elif stype == 'sum':
            self.title = 'Sum of Intensity inside objects'
        elif stype == 'std':
            self.title = 'Standard deviation of intensity inside objects'
        elif stype == 'var':
            self.title = 'Variance of intensity inside objects'
        elif stype == 'area':
            self.title = 'Area of objects (size in voxels)'
        elif stype == 'logarea':
            self.title = 'log10(Area) of objects (log size in voxels)'
        elif stype == 'bbox':
            self.title = 'Size of bounding box enclosing objects'
        elif stype == 'logbbox':
            self.title = 'log10(Size) of bounding box enclosing objects'
        elif stype == 'posx':
            self.title = 'Position (X) of the center of mass of the objects'
        elif stype == 'posy':
            self.title = 'Position (Y) of the center of mass of the objects'
        elif stype == 'posz':
            self.title = 'Position (Z) of the center of mass of the objects'

        self.feature = self.DM.load_ds('objects/{}'.format(stype))
        self.replot()

    def replot(self):
        self.mplcanvas.ax.clear()
        sns.distplot(self.feature, hist=False, kde=True, rug=False,
                     kde_kws={'lw': 3, 'shade': True, 'kernel': self.kernel},
                     ax=self.mplcanvas.ax)
        if self.labels is None:
            sns.rugplot(self.feature, ax=self.mplcanvas.ax)
        else:
            num_objects = self.num_objects
            for i in range(num_objects):
                l = self.labels[i]
                c = self.colors[l] if l >= 0 else '#0000FF'
                f = self.feature[i]
                self.mplcanvas.ax.axvline(f, 0, 0.05, lw=1, c=c)
        if self.selected_label is not None:
            f = self.feature[self.selected_label - 1]
            self.mplcanvas.ax.axvline(f, ymax=0.08, linewidth=3, color='#FF0000')
            self.mplcanvas.ax.set_title('Selectec value: {}'.format(f))
        self.mplcanvas.ax.set_xlabel(self.title)
        self.mplcanvas.redraw()

    def on_label_selected(self, label):
        self.selected_label = label
        self.replot()

    def on_export_plot(self):
        full_path = QtWidgets.QFileDialog.getSaveFileName(self, "Select output filename",
                                                      filter='*.png')
        if full_path is not None and len(full_path) > 0:
            if not full_path.endswith('.png'):
                full_path += '.png'
        self.mplcanvas.fig.savefig(full_path)

    def on_export_stats(self):
        features = FEATURE_TYPES

        total = len(features)

        if self.labels is not None:
            total += 1

        data = np.zeros((self.num_objects, total), np.float32)

        for n, ftype in enumerate(features):
            feature = self.DM.load_ds('objects/{}'.format(ftype))
            data[:, n] = feature[:]

        if self.labels is not None:
            features += ['class']
            data[:, -1] = self.labels

        full_path = QtWidgets.QFileDialog.getSaveFileName(self, "Select output filename",
                                                      filter='*.csv')
        if full_path is not None and len(full_path) > 0:
            if not full_path.endswith('.csv'):
                full_path += '.csv'
            df = pd.DataFrame(data, columns=features)
            df.to_csv(full_path)

    def on_kernel_changed(self):
        self.kernel = self.kernels.currentText()
        self.replot()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Escape \
                and self.selected_label is not None:
            self.selected_label = None
            self.label_canvas.select_object(None)
            self.replot()

    def on_select_object(self, ev):
        if ev.inaxes != self.mplcanvas.ax or self.feature is None:
            return

        x = ev.xdata
        self.selected_label = np.abs(self.feature[:] - x).argmin() + 1
        self.label_canvas.select_object(self.selected_label)
        self.replot()

    def on_rules_computed(self):
        self.labels = self.DM.load_ds('objects/objlabels')
        self.color_idx = list(set(np.unique(self.labels)) - set([-1]))
        self.colors = {i:self.label_rules.labels[i].colorbutton.color
                       for i in self.color_idx}
        self.label_canvas.colour(self.labels, self.color_idx, self.colors)
        self.replot()

    def reset(self):
        self.label_canvas.reset()
        self.label_rules.reset()

        self.objects = None
        self.selected_label = None
        self.feature = None
        self.colors = None
        self.color_idx = None
        self.labels = None
        self.computed_level = None
        self.mplcanvas.ax.clear()
        self.mplcanvas.redraw()


class LabelRule(QtWidgets.QWidget):

    deleted = QtCore.pyqtSignal(int)

    def __init__(self, idx, parent=None):
        super(LabelRule, self).__init__(parent=parent)

        self.idx = idx
        vbox = vbox = QtWidgets.QVBoxLayout()
        self.setLayout(vbox)

        delbutton = QtWidgets.QPushButton('X')
        delbutton.setMaximumWidth(30)
        label = QtWidgets.QLabel('Rule {}'.format(idx+1))
        self.fcombo = TComboBox('Feature:', FEATURE_OPTIONS)
        self.opcombo = TComboBox('', ['>', '<'])
        self.threshold = PLineEdit(0.5, parse=float)

        vbox.addWidget(HWidgets(delbutton, label, None, self.fcombo,
                                None, self.opcombo, None, self.threshold,
                                stretch=[0, 0, 1, 0, 1, 0, 1, 0]))

        delbutton.clicked.connect(self.on_delete)

    def on_delete(self):
        self.deleted.emit(self.idx)


class LabelOption(RoundedWidget):

    computed = QtCore.pyqtSignal(int, str, str, list)
    deleted = QtCore.pyqtSignal(int)
    compute_others = QtCore.pyqtSignal(int)

    def __init__(self, idx, parent=None):
        super(LabelOption, self).__init__(parent=parent)

        self.idx = idx
        vbox = QtWidgets.QVBoxLayout()
        self.setLayout(vbox)

        delbutton = QtWidgets.QPushButton('X')
        delbutton.setMaximumWidth(30)
        self.colorbutton = ColorButton()
        self.label = PLineEdit('Label {}'.format(idx+1), parse=str)
        self.label.setText('Label {}'.format(idx+1))
        rulebutton = QtWidgets.QPushButton('Add new rule')
        bapply = QtWidgets.QPushButton('Apply')
        othersbutton = QtWidgets.QPushButton('Select Others')
        vbox.addWidget(HWidgets(delbutton, self.colorbutton, self.label, None,
                                rulebutton, bapply, othersbutton,
                                stretch=[0, 0, 0, 1, 0]))

        rulebutton.clicked.connect(self.add_rule)
        delbutton.clicked.connect(self.on_delete_label)
        bapply.clicked.connect(self.on_apply)
        othersbutton.clicked.connect(self.on_apply_others)

        self.rules = {}
        self.total_tules = 0

    def add_rule(self):
        new_rule = LabelRule(self.total_tules)
        new_rule.deleted.connect(self.on_rule_deleted)
        self.layout().addWidget(new_rule)
        self.rules[self.total_tules] = new_rule
        self.total_tules += 1

    def on_rule_deleted(self, idx):
        self.rules[idx].setParent(None)
        del self.rules[idx]

    def on_delete_label(self):
        self.deleted.emit(self.idx)

    def on_apply(self):
        name = self.label.value()
        color = self.colorbutton.color
        rules = []
        for rule in self.rules.values():
            idx = rule.fcombo.currentIndex()
            smaller = rule.opcombo.currentIndex()
            thresh = rule.threshold.value()
            rules += [(idx, smaller, thresh)]

        if len(rules) > 0:
            self.computed.emit(self.idx, name, color, rules)

    def on_apply_others(self):
        self.compute_others.emit(self.idx)


class LabelRules(QtWidgets.QWidget):

    computed = QtCore.pyqtSignal()
    compute_others = QtCore.pyqtSignal(int)

    def __init__(self, parent=None):
        super(LabelRules, self).__init__(parent=parent)

        self.DM = DataModel.instance()
        self.setMinimumHeight(300)

        vbox = QtWidgets.QVBoxLayout()
        self.setLayout(vbox)

        self.save_labels = QtWidgets.QPushButton('Save labels')
        self.level_combo = SComboBox()
        self.add_label = QtWidgets.QPushButton('Add new label')
        vbox.addWidget(HWidgets(self.level_combo, self.save_labels,
                                None, self.add_label, stretch=[0, 0, 1, 0]))

        groupbox = QtWidgets.QWidget()
        self.form = QtWidgets.QFormLayout()
        groupbox.setLayout(self.form)

        scroll = QtWidgets.QScrollArea()
        scroll.setWidget(groupbox)
        scroll.setWidgetResizable(True)
        vbox.addWidget(scroll)

        self.LM = LayerManager.instance()
        self.LBLM = LabelManager.instance()
        self.LBLM.levelLoaded.connect(self.on_level_added)
        self.LBLM.levelAdded.connect(self.on_level_added)
        self.LBLM.levelRemoved.connect(self.on_level_removed)
        self.add_label.clicked.connect(self.on_add_label)
        self.save_labels.clicked.connect(self.on_save_labels)

        self.label_count = 0
        self.labels = {}
        self.levels = []
        self.rules_computed = None
        self.initialized = None

    def initialize(self):
        self.initialized = True

    def on_level_added(self, level):
        self.levels.append(level)
        self.level_combo.addItem('Level {}'.format(level))
        self.level_combo.setCurrentIndex(level)

    def on_level_removed(self, level):
        idx = self.levels.index(level)
        self.level_combo.removeItem(idx)
        del self.levels[idx]

    def on_add_label(self):
        if self.initialized is None:
            return
        new_label = LabelOption(self.label_count)
        new_label.deleted.connect(self.on_delete_label)
        new_label.computed.connect(self.on_compute)
        new_label.compute_others.connect(self.on_compute_others)
        self.form.addRow(new_label)
        self.labels[self.label_count] = new_label
        self.label_count += 1

    def on_delete_label(self, idx):
        self.labels[idx].setParent(None)
        del self.labels[idx]

        if self.DM.has_ds('objects/objlabels'):
            data = self.DM.load_ds('objects/objlabels')
            data[data == idx] = -1
            self.DM.write_dataset('objects/objlabels', data)
        self.computed.emit()

    def on_compute(self, idx, name, color, rules):
        num_objects = self.DM.attr('objects/objects', 'num_objects')
        features = ['objects/{}'.format(f) for f in FEATURE_TYPES]
        out_ds = 'objects/objlabels'

        if not self.DM.has_ds(out_ds):
            self.DM.create_empty_dataset(out_ds, (num_objects,), np.int32)

        Launcher.instance().run(ac.apply_rules, features=features, label=idx,
                                num_objects=num_objects, rules=rules, out_ds=out_ds,
                                caption='Applying Rules', cb=self.on_rules_computed)

    def on_rules_computed(self, labels):
        self.rules_computed = True
        self.computed.emit()

    def on_compute_others(self, idx):
        num_objects = self.DM.attr('objects/objects', 'num_objects')
        data = self.DM.load_ds('objects/objlabels')
        if data is None:
            return
        data[data < 0] = idx
        self.DM.write_dataset('objects/objlabels', data)
        self.computed.emit()

    def on_save_labels(self):
        if self.rules_computed is None:
            return
        level = self.levels[self.level_combo.currentIndex()]
        quit_msg = "Are you sure you want to save new labels in [Level {}]?".format(level)
        reply = QtWidgets.QMessageBox.question(self, 'Message', quit_msg,
                                           QtWidgets.QMessageBox.Yes,
                                           QtWidgets.QMessageBox.No)

        if reply != QtWidgets.QMessageBox.Yes:
            return

        launcher = Launcher.instance()
        launcher.setup('Saving Labels on [Level {}]'.format(level))
        log.info('+ Loading objects')
        labels = self.DM.load_ds('objects/objlabels')
        objlabels = np.r_[-1, labels]
        current = self.DM.load_ds('objects/objects')

        log.info('+ Loading annotatins')
        target_ds = 'annotations/annotations{}'.format(level)
        target = self.DM.load_ds(target_ds)
        shift_label = target.max() + 1
        mask = (current > 0) & (objlabels[current] >= 0)
        target[mask] = objlabels[current[mask]] + shift_label
        self.DM.write_dataset(target_ds, target)

        log.info('+ Adding label widgets')
        labels = np.unique(labels)

        for l in labels:
            if l == -1:
                continue
            labelobj = self.labels[l]
            label = l + shift_label
            name = labelobj.label.value()
            color = labelobj.colorbutton.color
            self.LBLM.loadLabel(level, label, name, color, True, -1, -1)

        self.LBLM.save(level)
        self.LM.update()
        launcher.cleanup()

    def reset(self):
        self.label_count = 0
        for label in self.labels.values():
            label.setParent(None)
        self.labels = {}
        self.rules_computed = None
        self.initialized = None


class LabelSplitter(Plugin):

    name = 'Label Splitter'

    def __init__(self, data=None, ptype=Plugin.Widget):
        super(LabelSplitter, self).__init__(ptype=ptype)

        splitter = QtWidgets.QSplitter()
        self.addWidget(splitter)

        self.label_canvas = LabelCanvas()
        splitter.addWidget(self.label_canvas)

        self.label_explorer = LabelExplorer(self.label_canvas)
        splitter.addWidget(self.label_explorer)
        self.initialized = False

    def on_tab_changed(self, i):
        if self.initialized == False:
            self.initialized = self.label_canvas.initialize()




import h5py as h5
import numpy as np

import logging as log

from ..core import DataModel
from ..lib._superpixels import slic_supervoxels
from ..lib._rag import _unique_rag3d

DM = DataModel.instance()

def create_supervoxels(dataset=None, compactness=1., sp_shape=(5,10,10),
                       spacing=(5., 1., 1.), out_sv=None,
                       out_svindex=None, out_svtable=None,
                       out_svedges=None, out_svweights=None):

    log.info('+ Loading data into memory')
    data = DM.load_slices(dataset)

    log.info('+ Computing supervoxels')
    svlabels = slic_supervoxels(data, compactness=compactness,
                                sp_shape=sp_shape, spacing=spacing,
                                gpu=DM.selected_gpu)
    total_sv = svlabels.max()+1
    log.info('+ {} supervoxels created'.format(total_sv))

    log.info('+ Writing supervoxels to disk {}'.format(svlabels.shape))
    params = {
        'compactness': compactness,
        'sp_shape': sp_shape,
        'spacing': spacing,
        'num_supervoxels': total_sv,
        'source': dataset,
        'active': True
    }
    DM.create_empty_dataset(out_sv, shape=DM.data_shape, dtype=svlabels.dtype)
    DM.write_slices(out_sv, svlabels, params=params)

    log.info('+ Generating lookup table')
    svravel = svlabels.ravel()
    sortindexes = svravel.argsort()
    sortlabels = svravel[sortindexes]
    splits = np.searchsorted(sortlabels, np.arange(total_sv))
    if data.shape != DM.data_shape: # its a ROI, remap indexes
        log.info('+ Remaping indexes to ROI')
        zz, yy, xx = np.mgrid[:data.shape[0], :data.shape[1], :data.shape[2]]
        zz += DM.active_roi[0].start
        yy += DM.active_roi[1].start
        xx += DM.active_roi[2].start
        seq = np.row_stack([zz.ravel(), yy.ravel(), xx.ravel()])
        idxtable = np.ravel_multi_index(seq, DM.data_shape)
        sortindexes = idxtable[sortindexes]

    log.info('+ Writing lookup table to disk {} {}'\
             .format(sortindexes.shape, splits.shape))

    DM.write_dataset(out_svindex, sortindexes, params=dict(active=True))
    DM.write_dataset(out_svtable, splits, params=dict(active=True))

    log.info('+ Generating supervoxel graph')
    edges = _unique_rag3d(svlabels, 6, total_sv)
    weights = np.ones(edges.shape[0], np.float32)

    log.info('+ Extracted {} unique edges'.format(edges.shape))
    DM.write_dataset(out_svedges, edges, params=dict(active=True))
    DM.write_dataset(out_svweights, weights, params=dict(active=True))

    return out_sv, total_sv, out_svindex, out_svtable, edges, weights
